package starj.toolkits.services;

import starj.coffer.InvokeInstruction;

public class PropagationManager extends PropagationOperation {
    private static PropagationManager instance = new PropagationManager();
    private Propagation[] propagations;

    private PropagationManager() {
        super("PropagationManager");
    }
 
    public void addPropagation(Propagation propagation) {
        Propagation[] propagations;
        if (this.propagations != null) {
            propagations = new Propagation[this.propagations.length + 1];
            System.arraycopy(this.propagations, 0, propagations, 0,
                    this.propagations.length);
        } else {
            propagations = new Propagation[1];
        }
        propagations[propagations.length - 1] = propagation;
        this.propagations = propagations;
    }

    public void propagate(InvokeInstruction call_site,
            MethodEntity new_method, ExecutionContext call_context) {
        if (this.propagations != null) {
            Propagation[] propagations = this.propagations;
            for (int i = 0; i < propagations.length; i++) {
                propagations[i].propagate(call_site,
                        new_method, call_context);
            }
        }
    }
    
    public void unpropagate(ExecutionContext context) {
        if (this.propagations != null) {
            Propagation[] propagations = this.propagations;
            for (int i = 0; i < propagations.length; i++) {
                propagations[i].unpropagate(context);
            }
        }
    }
    
    public static PropagationManager v() {
        return PropagationManager.instance;
    }
}
